/*
 * Copyright (c) 2021 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

#include <zephyr/kernel.h>
#include <stdio.h>
#include <zephyr/logging/log.h>
#include <date_time.h>
#include <nrf_modem_gnss.h>

#if defined(CONFIG_SLM_NRF_CLOUD)
#include <net/nrf_cloud.h>
#include <net/nrf_cloud_codec.h>
#include "slm_at_nrfcloud.h"
#endif /* CONFIG_SLM_NRF_CLOUD */

#include "slm_util.h"
#include "slm_at_host.h"

LOG_MODULE_REGISTER(slm_at_ag3352, CONFIG_SLM_LOG_LEVEL);

#define LOCATION_REPORT_MS 5000

#define SEC_PER_MIN		(60UL)
#define MIN_PER_HOUR	(60UL)
#define SEC_PER_HOUR	(MIN_PER_HOUR * SEC_PER_MIN)
#define HOURS_PER_DAY	(24UL)
#define SEC_PER_DAY	(HOURS_PER_DAY * SEC_PER_HOUR)
/* (6.1.1980 UTC - 1.1.1970 UTC) */
#define GPS_TO_UNIX_UTC_OFFSET_SECONDS	(315964800UL)
/* UTC/GPS time offset as of 1st of January 2017. */
#define GPS_TO_UTC_LEAP_SECONDS	(18UL)

/** #XGPS operations. */
enum slm_gnss_operation {
	GPS_STOP,
	GPS_START,
};

#include <modem/lte_lc.h>
//Peter add start
void at_test_cesq_ip_cat_nb1(void)
{
	int err_code;
	char ret_buff[60 + sizeof("OK\r\n")];
	const char *at_cmd_str;

	// //复位模块
	// at_cmd_str = "AT%%XFACTORYRESET=0";
	// printk("%s\r", at_cmd_str);
	// err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	// if (err_code) 
	// {
	// 	LOG_ERR("Not able to retrieve device ret_buff from modem");
	// }
	// else
	// {
	// 	printk("%s", ret_buff);
	// }

	//查询射频开关
	at_cmd_str = "AT+CFUN?";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		LOG_ERR("Not able to retrieve device ret_buff from modem");
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}

	//关闭射频
	at_cmd_str = "AT+CFUN=4";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		LOG_ERR("Not able to retrieve device ret_buff from modem");
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}

	//设置为CAT-NB1模式，不开GPS
	at_cmd_str = "AT%%XSYSTEMMODE=0,1,0,0";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		LOG_ERR("Not able to retrieve device ret_buff from modem");
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}

	//查询NB或CAT-M模式
	at_cmd_str = "AT%%XSYSTEMMODE?";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		//LOG_ERR("Not able to retrieve device ret_buff from modem");
		printk("%s", ret_buff);
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}

	//查询sim卡
	at_cmd_str = "AT%%XICCID";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		//LOG_ERR("Not able to retrieve device ret_buff from modem");
		printk("%s", ret_buff);
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}


	//打开射频
	at_cmd_str = "AT+CFUN=1";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		LOG_ERR("Not able to retrieve device ret_buff from modem");
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}
	k_sleep(K_MSEC(30000));

	//查询信号强度
	at_cmd_str = "AT+CESQ";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff)+20, at_cmd_str );
	if (err_code) 
	{
		//LOG_ERR("Not able to retrieve device ret_buff from modem");
		printk("%s", ret_buff);
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}

	//for( uint8_t i = 0; i < 10; i++ )
	{
		//查询IP地址
		at_cmd_str = "AT+CGDCONT?";
		printk("%s\r", at_cmd_str);
		err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff) + 20, at_cmd_str );
		if (err_code) 
		{
			//LOG_ERR("Not able to retrieve device ret_buff from modem");
			printk("%s", ret_buff);
			//return err_code;
		}
		else
		{
			printk("%s", ret_buff);
		}
		k_sleep(K_MSEC(15000));
	}

	k_sleep(K_MSEC(30000));
	printk("%s\r\n");

	//连接到nRF Cloud
	at_cmd_str = "AT#XNRFCLOUD=1";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		//LOG_ERR("Not able to retrieve device ret_buff from modem");
		printk("%s", ret_buff);
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}
	k_sleep(K_MSEC(25000));

	for( uint8_t i = 0; i < 10; i++ )
	{
		//查询nRF Cloud 连接情况
		at_cmd_str = "AT#XNRFCLOUD?";
		printk("%s\r", at_cmd_str);
		err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff) + 20, at_cmd_str );
		if (err_code) 
		{
			//LOG_ERR("Not able to retrieve device ret_buff from modem");
			printk("%s", ret_buff);
			//return err_code;
		}
		else
		{
			printk("%s", ret_buff);
		}
		k_sleep(K_MSEC(5000));
	}

}
//Peter add end

void cmd_test(void)
{
	int err = -EINVAL;
	uint16_t cmd1;
	uint32_t cmd2;
	char cmd3[128] = {0};
	size_t cmd3_len;
	
	uint32_t param_count = at_params_valid_count_get(&slm_at_param_list);  //判断参数个数
	err = at_params_unsigned_short_get(&slm_at_param_list, 1, &cmd1);  //读取第一个参数
	err = at_params_unsigned_int_get(&slm_at_param_list, 3, &cmd2);  //读取第一个参数
	err = at_params_string_get(&slm_at_param_list, 2, cmd3, &cmd3_len);  //读取一个字符串
	LOG_INF(" err = %d\r\n", err);
	
	LOG_INF("param_count = %d\r\n", param_count );
	LOG_INF("cmd1 = %d\r\n",cmd1);
	LOG_INF("cmd2 = %d\r\n",cmd2);
	LOG_INF(" cmd3 = %s\r\n", cmd3);

	//发一组数据到nRF Cloud
	int activated = 0;
	int cfun_mode = 0;

	int ret = 0;
	ret = nrf_modem_at_scanf("AT%XSYSTEMMODE?","%XSYSTEMMODE: %*d,%*d,%d", &activated);
	/*parse %XSYSTEMMODE=<LTE_M_support>,<NB_IoT_support>,<GNSS_support>,<LTE_preference> */
	if ( ret == 1) 
	{
		if (activated == 0) 
		{
			LOG_INF("GPS not activated\r\n");
		}
		else
		{
			LOG_INF("GPS activated\r\n");
		}
	}

	LOG_INF(" activated = %d\r\n", activated);
	LOG_INF(" ret = %d\r\n", ret);

	//发送at+CFUN=1
	#if 1
	//at_test_cesq_ip_cat_nb1();
	int err_code;
	char ret_buff[60 + sizeof("OK\r\n")];
	const char *at_cmd_str;

	//查询射频开关
	at_cmd_str = "AT+CFUN?";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		LOG_ERR("Not able to retrieve device ret_buff from modem");
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}

	//关闭射频
	at_cmd_str = "AT+CFUN=4";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		LOG_ERR("Not able to retrieve device ret_buff from modem");
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}

	//设置为CAT-NB1模式，不开GPS
	at_cmd_str = "AT%%XSYSTEMMODE=0,1,0,0";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		LOG_ERR("Not able to retrieve device ret_buff from modem");
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}

	//查询NB或CAT-M模式
	at_cmd_str = "AT%%XSYSTEMMODE?";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		//LOG_ERR("Not able to retrieve device ret_buff from modem");
		printk("%s", ret_buff);
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}

	//查询sim卡
	at_cmd_str = "AT%%XICCID";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		//LOG_ERR("Not able to retrieve device ret_buff from modem");
		printk("%s", ret_buff);
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}


	//打开射频
	at_cmd_str = "AT+CFUN=1";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		LOG_ERR("Not able to retrieve device ret_buff from modem");
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}
	k_sleep(K_MSEC(30000));

	//查询信号强度
	at_cmd_str = "AT+CESQ";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff)+20, at_cmd_str );
	if (err_code) 
	{
		//LOG_ERR("Not able to retrieve device ret_buff from modem");
		printk("%s", ret_buff);
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}

	//for( uint8_t i = 0; i < 10; i++ )
	{
		//查询IP地址
		at_cmd_str = "AT+CGDCONT?";
		printk("%s\r", at_cmd_str);
		err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff) + 20, at_cmd_str );
		if (err_code) 
		{
			//LOG_ERR("Not able to retrieve device ret_buff from modem");
			printk("%s", ret_buff);
			//return err_code;
		}
		else
		{
			printk("%s", ret_buff);
		}
		k_sleep(K_MSEC(5000));
	}

	k_sleep(K_MSEC(15000));
	//连接到nRF Cloud
	at_cmd_str = "AT#XNRFCLOUD=1";
	printk("%s\r", at_cmd_str);
	err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff), at_cmd_str );
	if (err_code) 
	{
		//LOG_ERR("Not able to retrieve device ret_buff from modem");
		printk("%s", ret_buff);
		//return err_code;
	}
	else
	{
		printk("%s", ret_buff);
	}
	k_sleep(K_MSEC(5000));

	for( uint8_t i = 0; i < 10; i++ )
	{
		//查询nRF Cloud 连接情况
		at_cmd_str = "AT#XNRFCLOUD?";
		printk("%s\r", at_cmd_str);
		err_code = nrf_modem_at_cmd( ret_buff, sizeof(ret_buff) + 20, at_cmd_str );
		if (err_code) 
		{
			//LOG_ERR("Not able to retrieve device ret_buff from modem");
			printk("\r\n%s", ret_buff);
			//return err_code;
		}
		else
		{
			printk("%s", ret_buff);
		}
		k_sleep(K_MSEC(5000));
	}

	#endif

	/*parse +CFUN: <fun> */
	if (nrf_modem_at_scanf("AT+CFUN?", "+CFUN: %d", &cfun_mode) == 1) 
	{
		if (cfun_mode == 1 || cfun_mode == 31) 
		{
			LOG_INF("LTE activated\r\n");
		}
		else
		{
			LOG_INF("LTE not activated\r\n");
		}
	}

	LOG_INF(" cfun_mode = %d\r\n", cfun_mode);
}

#if defined(CONFIG_SLM_NRF_CLOUD)
//发送数据到nRF Cloud
static int do_cloud_send_obj(struct nrf_cloud_obj *const obj)
{
	int err;
	struct nrf_cloud_tx_data msg = {
		.obj = obj,
		.topic_type = NRF_CLOUD_TOPIC_MESSAGE,
		.qos = MQTT_QOS_0_AT_MOST_ONCE
	};

	err = nrf_cloud_send(&msg);
	if (err) {
		LOG_ERR("nrf_cloud_send failed, error: %d", err);
	}

	(void)nrf_cloud_obj_free(obj);

	return err;
}

//发送定位信息到nRF Cloud
static void send_location(struct nrf_modem_gnss_pvt_data_frame * const pvt_data)
{
	static int64_t last_ts_ms = NRF_CLOUD_NO_TIMESTAMP;
	int err;
	struct nrf_cloud_gnss_data gnss = {
		.ts_ms = NRF_CLOUD_NO_TIMESTAMP,
		.type = NRF_CLOUD_GNSS_TYPE_MODEM_PVT,
		.mdm_pvt = pvt_data
	};
	NRF_CLOUD_OBJ_JSON_DEFINE(msg_obj);

	/* On failure, NRF_CLOUD_NO_TIMESTAMP is used and the timestamp is omitted */
	(void)date_time_now(&gnss.ts_ms);

	if ((last_ts_ms == NRF_CLOUD_NO_TIMESTAMP) ||
	    (gnss.ts_ms == NRF_CLOUD_NO_TIMESTAMP) ||
	    (gnss.ts_ms > (last_ts_ms + LOCATION_REPORT_MS))) {
		last_ts_ms = gnss.ts_ms;
	} else {
		return;
	}

	/* Encode the location data into a device message */
	err = nrf_cloud_obj_gnss_msg_create(&msg_obj, &gnss);
	if (!err) {
		err = do_cloud_send_obj(&msg_obj);
	}

	if (err) {
		LOG_WRN("Failed to send location, error %d", err);
	}
}
#endif /* CONFIG_SLM_NRF_CLOUD */



// $GNGGA  //GPS定位信息
// $GNGLL  //定位地理信息
// $GNGSA  //当前卫星信息
// $GNGSA  //当前卫星信息
// $GNGSA  //当前卫星信息
// $GNGSA  //当前卫星信息
// $GPGSV  //可见卫星数
// $GLGSV  //可见卫星数
// $GAGSV  //可见卫星数
// $GBGSV  //可见卫星数
// $GNRMC  //推荐定位信息
// $GNVTG  //地面速度信息
// $GNZDA  //当前时间信息

//AG3352一组GNSS数据示例
const char ag3352_gnss_data1[] = 
{	
"\
$GNGGA,094117.000,2232.5506,N,11357.0824,E,1,7,2.80,47.2,M,-2.5,M,,*66\r\n\
$GNGLL,2232.5506,N,11357.0824,E,094117.000,A,A*45\r\n\
$GNGSA,A,3,32,23,,,,,,,,,,,2.95,2.80,0.92,1*0E\r\n\
$GNGSA,A,3,68,78,67,,,,,,,,,,2.95,2.80,0.92,2*0D\r\n\
$GNGSA,A,3,,,,,,,,,,,,,2.95,2.80,0.92,3*0C\r\n\
$GNGSA,A,3,19,22,,,,,,,,,,,2.95,2.80,0.92,4*03\r\n\
$GPGSV,1,1,04,32,61,033,47,23,13,160,35,194,60,089,,196,,,43,1*55\r\n\
$GLGSV,1,1,03,68,69,040,47,78,45,015,41,67,37,126,46,1*40\r\n\
$GAGSV,1,1,01,08,,,40,7*7E\r\n\
$GBGSV,1,1,03,19,28,139,43,22,82,114,39,36,,,43,1*7D\r\n\
$GNRMC,094117.000,A,2232.5506,N,11357.0824,E,0.32,74.74,180923,,,A,V*38\r\n\
$GNVTG,74.74,T,,M,0.32,N,0.59,K,A*1E\r\n\
$GNZDA,094117.000,18,09,2023,,*41\r\n\
"
};

const char ag3352_gnss_data2[] = 
{	
"\
$GNGGA,094130.000,2232.5471,N,11357.0816,E,1,8,2.79,53.9,M,-2.5,M,,*64\r\n\
$GNGLL,2232.5471,N,11357.0816,E,094130.000,A,A*40\r\n\
$GNGSA,A,3,32,23,,,,,,,,,,,2.94,2.79,0.92,1*09\r\n\
$GNGSA,A,3,68,78,67,,,,,,,,,,2.94,2.79,0.92,2*0A\r\n\
$GNGSA,A,3,08,,,,,,,,,,,,2.94,2.79,0.92,3*03\r\n\
$GNGSA,A,3,19,22,,,,,,,,,,,2.94,2.79,0.92,4*04\r\n\
$GPGSV,2,1,05,32,61,033,47,23,13,160,35,10,,,36,194,60,089,,1*6A\r\n\
$GPGSV,2,2,05,196,,,43,1*58\r\n\
$GLGSV,1,1,03,68,69,040,47,78,45,015,43,67,37,126,45,1*41\r\n\
$GAGSV,1,1,01,08,61,024,40,7*4F\r\n\
$GBGSV,1,1,03,19,28,139,43,22,82,114,40,36,,,43,1*73\r\n\
$GNRMC,094130.000,A,2232.5471,N,11357.0816,E,0.26,244.04,180923,,,A,V*0E\r\n\
$GNVTG,244.04,T,,M,0.26,N,0.48,K,A*2D\r\n\
$GNZDA,094130.000,18,09,2023,,*44\r\n\
"
};

//获取到一帧GNSS buffer所占最大缓存
//声明2个GNSS buffer 单buffer最大为1024字节
char at3352_gnss_buffer[2][1024];
char *p_at3352_gnss_buffer;
uint8_t at3352_gnss_buffer_state = 0;
uint8_t at3352_gnss_buffer_flag = 0;
//将串口数据存到GNSS buffer
void slm_at_read_ag3352_gnss_data(void)
{
	rsp_send("\r\n read NRF_MODEM_GNSS_DATA_PVT from uart\r\n");
	if( !at3352_gnss_buffer_flag )
	{
		at3352_gnss_buffer_flag = 1;
		//at3352_gnss_buffer[0][1024];
		p_at3352_gnss_buffer = ag3352_gnss_data1;
	}
	else
	{
		at3352_gnss_buffer_flag = 0;
		//at3352_gnss_buffer[1][1024];
		p_at3352_gnss_buffer = ag3352_gnss_data2;
	}

	//p_at3352_gnss_buffer = at3352_gnss_buffer[at3352_gnss_buffer_state];
	//at3352_gnss_buffer_flag = 1;
}

#if 0
//纬度，单位为度  
pvt->latitude,  

//经度，单位为度  
pvt->longitude,  

//相对于WGS-84椭球体的海拔高度，单位为米  
pvt->altitude,  

//位置精度（2D 1-sigma），单位为米  
pvt->accuracy,  

//高度精度（1-sigma），单位为米  
pvt->altitude_accuracy,  

//水平速度，单位为米/秒  
pvt->speed,  

//速度精度（1-sigma），单位为米/秒  
pvt->speed_accuracy,  

//垂直速度，单位为米/秒。正值表示向上  
pvt->vertical_speed,  

//垂直速度精度（1-sigma），单位为米/秒  
pvt->vertical_speed_accuracy,  

//用户移动的航向，单位为度  
pvt->heading,  

//航向精度（1-sigma），单位为度  
pvt->heading_accuracy,  

//日期和时间  
pvt->nrf_modem_gnss_datetime datetime,  

//位置精度衰减  
pvt->pdop,  

//水平精度衰减  
pvt->hdop,  

//垂直精度衰减  
pvt->vdop,  

//时间精度衰减  
pvt->tdop,  

//请参阅 @ref nrf_modem_gnss_pvt_flag_bitmask  
pvt->flags,  

//描述了用于测量的最多12颗卫星
pvt->nrf_modem_gnss_sv sv[NRF_MODEM_GNSS_MAX_SATELLITES],  

//自上次启动以来的累积GNSS执行时间，单位为毫秒  
pvt->execution_time,
#endif

#include <stdio.h>
#include <string.h>
typedef struct __f_gnss_data
{
	char *pos;  //数据位置
	uint16_t size;
} f_gnss_data;


//转换一帧GNSS数据为pvt数据
int ag3352_gnss_data_pvt_data( char *p_gnss_buffer, struct nrf_modem_gnss_pvt_data_frame *pvt )
{
	int err = -EINVAL;
	
	#if 1
	//提取以$G开头和以*结尾的有效GNSS数据位置和长度，并保存到结构体f_gnss_data中
	#define POS_GNSS_HEAD_MAX 16  //最多保存32条有效数据
	f_gnss_data gnss_buffer[POS_GNSS_HEAD_MAX];  //存储有效GNSS数据，$G起始地址和数据长度
	//char *pos = (char*)ag3352_gnss_data1;  //设置起始地址，查询$G
	char *pos = p_gnss_buffer;  //设置起始地址，查询$G
	char *ptr = NULL;  //查询*
	char *next_pos = NULL;  //查询第二个$G
	uint8_t idx = 0;  //有效数据计数
	
    while ( ( pos = strstr( pos, "$G" ) ) != NULL ) 
	{  
		ptr = strstr( pos, "*" ); 
		next_pos = strstr( pos + 1, "$G" );

		//查找到有效数据，后边还有数据
		if( ptr != NULL && next_pos != NULL && (ptr - pos) > 0 && (next_pos - ptr) > 0 )
		{
			gnss_buffer[idx].pos = pos;
			gnss_buffer[idx].size = ptr - pos + 5;

			//LOG_INF( "gnss[%d] size = %d data = %.*s\r\n", idx, gnss_buffer[idx].size, gnss_buffer[idx].size, gnss_buffer[idx].pos );

			idx++;  //有效数据索引+1

			//处理完本组数据后还有数据
			pos = next_pos;
		}
		//查找到有效数据，是最后一组数据
		else if( ptr != NULL && next_pos == NULL && (ptr - pos) > 0 )
		{
			gnss_buffer[idx].pos = pos;
			gnss_buffer[idx].size = ptr - pos + 5;

			//LOG_INF( "gnss data[%d] = %.*s\r\n", idx, gnss_buffer[idx].size, gnss_buffer[idx].pos );

			idx++;  //有效数据索引+1

			//最后一组数据，处理完后跳出
			break;
		}
		//无效数据
		else
		{
			LOG_INF("none used data\r\n");
		}

		//超过要保存的最大数据，则直接退出
		if( idx >= POS_GNSS_HEAD_MAX )
		{
			break;
		}
    }
	#endif

	#if 1  //解析GPS数据开始
	//读取串口有效$GNSS数据，格式化为NRF_MODEM_GNSS_DATA_PVT类型，即解析到pvt数据结构，再上传到nRF Cloud
	char *p_buffer;  //要查询的数据头位置
	uint16_t bufer_length;  //每条数据的长度
	memset( pvt, 0, sizeof(pvt) );
	memset( pvt->sv, 0, sizeof(pvt->sv) );
	uint8_t fix_star_id = 0;  //第几个卫星数据
	uint8_t fix_star_id2 = 0;  //第几个卫星数据
	int fix_star_cnt = NRF_MODEM_GNSS_MAX_SATELLITES;  //用来定位的卫星数量，默认设置到最大

	for( uint8_t i = 0; i < idx; i ++ )
	{
		p_buffer = gnss_buffer[i].pos;
		bufer_length = gnss_buffer[i].size;
		//LOG_INF( "pos[%d] = %d buffer[%d] = %.*s\r\n", i, p_buffer[i], i, 6, p_buffer );  //打印一条GNSS数据头位置和数据长度
		LOG_INF( "buffer[%d] length = %d data = %.*s\r\n", i, bufer_length, bufer_length, p_buffer );  //打印一条完整GNSS数据

		//GPS定位信息
		//$GNGGA,094117.000,2232.5506,N,11357.0824,E,1,7,2.80,47.2,M,-2.5,M,,*66/r/n
		//$GNGGA 语句的基本格式如下（其中 M 指单位 M， hh 指校验和， CR 和 LF 代表回车换行，下同）：
		//$GNGGA,(1),(2),(3),(4),(5),(6),(7),(8),(9),M,(10),M,(11),(12)*hh(CR)(LF)
		//(1) UTC 时间，格式为 hhmmss.ss；(2) 纬度，格式为 ddmm.mmmmm（度分格式）；(3) 纬度半球， N 或 S（北纬或南纬）；(4) 经度，格式为 dddmm.mmmmm（度分格式）；(5) 经度半球， E 或 W（东经或西经）；
		//(6) GPS 状态， 0=未定位， 1=非差分定位， 2=差分定位；(7) 正在使用的用于定位的卫星数量（00~12）(8) HDOP 水平精确度因子（0.5~99.9）(9) 海拔高度（-9999.9 到 9999.9 米）(10) 大地水准面高度（-9999.9 到 9999.9 米）
		//(11) 差分时间（从最近一次接收到差分信号开始的秒数，非差分定位，此项为空）(12) 差分参考基站标号（0000 到 1023， 首位 0 也将传送，非差分定位，此项为空)
		if ( strncmp( p_buffer, "$GNGGA", 6) == 0 )
		{
			#if 1
			double latitude = 0;  //纬度，单位为度
			char latitude_flag = 0;  //纬度半球 N（北半球）或 S（南半球）
			double longitude = 0;  //经度，单位为度
			char longitude_flag = 0;  //经度半球 E（东经）或 W（西经）
			char fix_state = 0;  //定位状态， A=有效定位， V=无效定位
			float hdop = 0;  //水平精度衰减
			float altitude = 0;  //相对于WGS-84椭球体的海拔高度，单位为米
			float geodi = 0;  //大地水准面高度
			sscanf(p_buffer, "$GNGGA,%*lf,%lf,%c,%lf,%c,%d,%d,%f,%f,M,%f,%*s", &latitude, &latitude_flag, &longitude, &longitude_flag, &fix_state, &fix_star_cnt, &hdop, &altitude, &geodi);
			
			LOG_INF( "latitude = %10.5lf\r\n", latitude);
			LOG_INF( "latitude_flag = %c\r\n", latitude_flag);
			LOG_INF( "longitude = %10.5lf\r\n", longitude);
			LOG_INF( "longitude_flag = %c\r\n", longitude_flag);
			LOG_INF( "fix_state = %d\r\n", fix_state);
			LOG_INF( "fix_star_cnt = %d\r\n", fix_star_cnt );
			LOG_INF( "hdop = %5.2f\r\n", hdop);
			LOG_INF( "altitude = %7.1f\r\n", altitude);
			LOG_INF( "geodi = %7.1f\r\n", geodi);

			//填充数据到nRF Cloud结构体pvt
			pvt->latitude = latitude;
			pvt->longitude = longitude;
			pvt->hdop = hdop;
			pvt->altitude = altitude;
			#endif

			# if 1
			//解析UTC时间
			int hour, minute, second, millisecond;
			sscanf(p_buffer, "$GNGGA,%02d%02d%02d.%03d", &hour, &minute, &second, &millisecond);
			LOG_INF( "utc time = %02d:%02d:%02d:%03d\r\n", hour, minute, second, millisecond );

			//填充数据到nRF Cloud结构体pvt
			pvt->datetime.hour = (uint8_t)hour;
			pvt->datetime.minute = (uint8_t)minute;
			pvt->datetime.seconds = (uint8_t)second;
			pvt->datetime.ms = (uint16_t)millisecond;
			#endif
		}
		//$GNGLL  //定位地理信息
		//$GNGLL,2232.5506,N,11357.0824,E,094117.000,A,A*45\r\n
		//$GNGLL 语句的基本格式如下：$GNGLL,(1),(2),(3),(4),(5),(6),(7)*hh(CR)(LF)
		//(1) 纬度 ddmm.mmmmm（度分）(2) 纬度半球 N（北半球）或 S（南半球）(3) 经度 dddmm.mmmmm（度分）(4) 经度半球 E（东经）或 W（西经）(5) UTC 时间：hhmmss（时分秒）
		//(6) 定位状态， A=有效定位， V=无效定位(7) 模式指示（A=自主定位， D=差分， E=估算， N=数据无效）举例如下：$GNGLL,2318.1330,N,11319.7250,E,095556.000,A,A*4F
		else if ( strncmp( p_buffer, "$GNGLL", 6) == 0 )
		{
			#if 1
			//解析经纬度
			double latitude = 0;  //纬度
			char latitude_flag = 0;  //纬度半球 N（北半球）或 S（南半球）
			double longitude = 0;  //经度
			char longitude_flag = 0;  //经度半球 E（东经）或 W（西经）
			char fix_state = 0;  //定位状态， A=有效定位， V=无效定位
			char fix_mode = 0;  //定位模式 A=自主定位， D=差分， E=估算， N=数据无效

			sscanf(p_buffer, "$GNGLL,%lf,%c,%lf,%c,%*lf,%c,%c,%*s", &latitude, &latitude_flag, &longitude, &longitude_flag, &fix_state, &fix_mode  );
			LOG_INF( "latitude = %10.4lf\r\n", latitude);
			LOG_INF( "latitude_flag = %c\r\n", latitude_flag);
			LOG_INF( "longitude = %10.4lf\r\n", longitude);
			LOG_INF( "longitude_flag = %c\r\n", longitude_flag);
			LOG_INF( "fix_state = %c", fix_state );
			LOG_INF( "fix_mode = %c", fix_mode );

			//填充数据到nRF Cloud结构体
			pvt->latitude = latitude;
			pvt->longitude = longitude;
			#endif

			# if 1
			//解析时间
			int hour, minute, second, millisecond;
			sscanf(p_buffer, "$GNGLL,%*lf,%*c,%*lf,%*c,%02d%02d%02d.%03d", &hour, &minute, &second, &millisecond);
			LOG_INF( "utc time = %02d:%02d:%02d:%03d\r\n", hour, minute, second, millisecond );

			//填充数据到nRF Cloud结构体
			pvt->datetime.hour = (uint8_t)hour;
			pvt->datetime.minute = (uint8_t)minute;
			pvt->datetime.seconds = (uint8_t)second;
			pvt->datetime.ms = (uint16_t)millisecond;
			#endif
		}
		//$GNGSA  //当前卫星信息
		//$GNGSA,A,3,32,23,,,,,,,,,,,2.95,2.80,0.92,1*0E\r\n
		//$GNGSA,A,3,68,78,67,,,,,,,,,,2.95,2.80,0.92,2*0D\r\n
		//$GNGSA,A,3,,,,,,,,,,,,,2.95,2.80,0.92,3*0C\r\n
		//$GNGSA,A,3,19,22,,,,,,,,,,,2.95,2.80,0.92,4*03\r\n
		//$GNGSA 语句的基本格式如下：$GNGSA,(1),(2),(3),(3),(3),(3),(3),(3),(3),(3),(3),(3),(3),(3),(4),(5),(6)*hh(CR)(LF)
		//(1) 模式， M = 手动， A = 自动。(2) 定位类型， 1=未定位， 2=2D 定位， 3=3D 定位。(3) 正在用于定位的卫星号（01~32）(4) PDOP 综合位置精度因子（0.5-99.9）ALIENTEK(5) HDOP 水平精度因子 1（0.5-99.9）
		//(6) VDOP 垂直精度因子（0.5-99.9）注 1：精度因子值越小，则准确度越高。
		else if ( strncmp( p_buffer, "$GNGSA", 6) == 0 )
		{
			char fix_auto;  //定位模式 A=自主定位， D=差分， E=估算， N=数据无效
			uint16_t signal = 0;  //定位类型， 1=未定位， 2=2D 定位， 3=3D 定位
			sscanf( p_buffer, "$GNGSA,%c,%d,%*s", &fix_auto, &signal );  //对应卫星的定位类型
			LOG_INF( "fix_auto = %c\r\n", fix_auto );
			LOG_INF( "signal = %d\r\n", signal );

			#if 1
			//获取定位使用的卫星ID
			uint16_t sv = 0;
			char *p_start = p_buffer + 10;
			char *p_end = strstr( p_buffer, "." );
			if( p_end != NULL )
			{
				p_end -= 2;
			}
			
			while( (p_end - p_start) > 0 )
			{
				p_start = strstr( p_start, "," );
				if( p_start != NULL && (p_start - p_end) > 0 )
				{
					break;
				}

				if( p_start[0] == ',' && p_start[1] != ',')
				{
					sscanf( p_start, ",%d,%*s", &sv );  //使用了那一颗卫星
					LOG_INF( "sv = %d\r\n", sv );
					
					pvt->sv[fix_star_id2].sv = sv;
					pvt->sv[fix_star_id2].signal = signal;
					fix_star_id2++;
					if( fix_star_id2 >= fix_star_cnt )
					{
						fix_star_id2 = 0;
					}
				}
				p_start++;
			}
			#endif

			#if 1
			//获取精度参数
			float pdop = 0;  //位置精度衰减
			float hdop = 0;  //水平精度衰减
			float vdop = 0;  //垂直精度衰减
			//float tdop = 0;  //时间精度衰减

			//定位数据起始位置为浮点数的第一个特殊符号'.'
			char *p_temp = strstr( p_buffer, "." );
			if( p_temp != NULL )
			{
				sscanf(p_temp - 1, "%f,%f,%f,%*s", &pdop, &hdop, &vdop);
			}
			LOG_INF("pdop: %4.2f\n", pdop);
			LOG_INF("hdop: %4.2f\n", hdop);
			LOG_INF("vdop: %4.2f\n", vdop);

			//填充数据到nRF Cloud结构体pvt
			pvt->pdop = pdop;
			pvt->hdop = hdop;
			pvt->vdop = vdop;
			#endif
		}
		//$GPGSV  //可见卫星数
		//$GPGSV,1,1,04,32,61,033,47,23,13,160,35,194,60,089,,196,,,43,1*55\r\n
		//$GLGSV,1,1,03,68,69,040,47,78,45,015,41,67,37,126,46,1*40\r\n
		//$GAGSV,1,1,01,08,,,40,7*7E\r\n
		//$GBGSV,1,1,03,19,28,139,43,22,82,114,39,36,,,43,1*7D\r\n
		//$GPGSV 语句的基本格式如下：$GPGSV, (1),(2),(3),(4),(5),(6),(7),...,(4),(5),(6),(7)*hh(CR)(LF)
		//(1) GSV 语句总数。(2) 本句 GSV 的编号。(3) 可见卫星的总数（00~12，前面的 0 也将被传输）。(4) 卫星编号（01~32，前面的 0 也将被传输）。(5) 卫星仰角（00~90 度，前面的 0 也将被传输）。
		//(6) 卫星方位角（000~359 度，前面的 0 也将被传输）(7) 信噪比（00~99dB，没有跟踪到卫星时为空）。注：每条 GSV 语句最多包括四颗卫星的信息，其他卫星的信息将在下一条$GPGSV 语句中输出。
		else if ( strncmp( p_buffer, "$GPGSV", 6) == 0 || strncmp( p_buffer, "$GLGSV", 6) == 0 || strncmp( p_buffer, "$GAGSV", 6) == 0 || strncmp( p_buffer, "$GBGSV", 6) == 0 )
		{
			#if 1
			uint16_t pid_max = 0;  //GSV 语句总数
			uint16_t pid = 0;  //本句 GSV 的编号
			uint16_t sv = 0;  //卫星id编号
			uint16_t cn0 = 0;  //卫星信噪比
			int16_t elevation = 0;  //卫星仰角
			int16_t azimuth = 0;  //卫星方位角

			//获取句柄总数 当权句柄编号 本句可见卫星总数
			uint16_t visible_star_cnt = 0;
			sscanf( p_buffer, "%*[^,],%d,%d,%d,%*s", &pid_max, &pid, &visible_star_cnt );
			LOG_INF( "pid_max = %d\r\n", pid_max );
			LOG_INF( "pid = %d\r\n", pid );
			LOG_INF( "Visible star = %d\r\n", visible_star_cnt);

			//依次处理本句可见卫星数据
			char *p_start = p_buffer + 13;
			char *p_end = strstr( p_buffer, "*" );
			if( p_end != NULL )
			{
				p_end -= 2;
			}
			while( (p_end - p_start) > 0 )
			{
				//处理完可见卫星数据后退出
				if( !visible_star_cnt )
				{
					break;
				}
				//处理可见卫星数据
				else
				{
					visible_star_cnt--;
				}

				//初始化要存放数据的变量
				sv = 0;
				cn0 = 0;
				elevation = 0;
				azimuth = 0;

				//(4)卫星编号 sv = 3
				p_start = strstr( p_start, "," );
				LOG_INF( "p_start = %.*s\r\n", 12, p_start + 1);  //打印本组卫星数据
				if( p_start == NULL || (p_start - p_end) > 0 )
				{
					break;
				}
				if( p_start[0] == ',' && p_start[1] != ',')
				{
					sscanf( p_start, ",%d,%*s", &sv );
					LOG_INF( "sv = %02d\r\n", sv );
				}

				//(5)卫星仰角 elevation = 61
				p_start++;
				p_start = strstr( p_start, "," );
				if( p_start == NULL || (p_start - p_end) > 0 )
				{
					break;
				}
				if( p_start[0] == ',' && p_start[1] != ',')
				{
					sscanf( p_start, ",%02d,%*s", &elevation );
					LOG_INF( "elevation = %02d\r\n", elevation );
				}

				//(6)卫星方位角 azimuth = 033
				p_start++;
				p_start = strstr( p_start, "," );
				if( p_start == NULL || (p_start - p_end) > 0 )
				{
					break;
				}
				if( p_start[0] == ',' && p_start[1] != ',')
				{
					sscanf( p_start, ",%03d,%*s", &azimuth );
					LOG_INF( "azimuth = %03d\r\n", azimuth );
				}

				//(7)信噪比 c/n0 = 47
				p_start++;
				p_start = strstr( p_start, "," );
				if( p_start == NULL || (p_start - p_end) > 0 )
				{
					break;
				}
				if( p_start[0] == ',' && p_start[1] != ',')
				{
					sscanf( p_start, ",%02d,%*s", &cn0 );
					LOG_INF( "c/n0 = %02d\r\n", cn0 );
				}

				//定位数据完整，切换到下个卫星
				if( sv && elevation && azimuth && cn0 )
				{
					//LOG_INF("GPS data ok\r\n");
					pvt->sv[fix_star_id].flags |= NRF_MODEM_GNSS_SV_FLAG_USED_IN_FIX;  //数据完整，则认为卫星数据健康，则用其进行定位
					pvt->sv[fix_star_id].sv = sv;
					pvt->sv[fix_star_id].elevation = elevation;
					pvt->sv[fix_star_id].azimuth = azimuth;
					pvt->sv[fix_star_id].cn0 = cn0;
					pvt->sv[fix_star_id].signal = 3;

					//提取有效卫星数据次数，不超过有效卫星数量。
					fix_star_id++;
					if( fix_star_id >= fix_star_cnt )
					{
						fix_star_id = fix_star_cnt;
					}
				}
				//定位数据不完整，剔除无效数据
				else
				{
					//LOG_INF("GPS data not ok\r\n");
					pvt->sv[fix_star_id].flags = 0;  //.flags |= NRF_MODEM_GNSS_SV_FLAG_UNHEALTHY;  //信号质量差
					pvt->sv[fix_star_id].sv = 0;
					pvt->sv[fix_star_id].elevation = 0;
					pvt->sv[fix_star_id].azimuth = 0;
					pvt->sv[fix_star_id].cn0 = 0;
					pvt->sv[fix_star_id].signal = 0;
				}
				p_start++;
			}
			#endif
		}
		// $GNRMC  //推荐定位信息
		//$GNRMC,094117.000,A,2232.5506,N,11357.0824,E,0.32,74.74,180923,,,A,V*38\r\n
		//$GNRMC 语句的基本格式如下：$GNRMC,(1),(2),(3),(4),(5),(6),(7),(8),(9),(10),(11),(12)*hh(CR)(LF)
		//(1) UTC 时间， hhmmss（时分秒）(2) 定位状态， A=有效定位， V=无效定位(3) 纬度 ddmm.mmmmm（度分）(4) 纬度半球 N（北半球）或 S（南半球）ALIENTEK(5) 经度 dddmm.mmmmm（度分）
		//(6) 经度半球 E（东经）或 W（西经）(7) 地面速率（000.0~999.9 节）(8) 地面航向（000.0~359.9 度，以真北方为参考基准）(9) UTC 日期， ddmmyy（日月年）(10) 磁偏角（000.0~180.0 度，前导位数不足则补 0）
		//(11) 磁偏角方向， E（东）或 W（西）(12) 模式指示（A=自主定位， D=差分， E=估算， N=数据无效）
		else if ( strncmp( p_buffer, "$GNRMC", 6) == 0 )
		{
			#if 1
			double latitude = 0;  //纬度
			char latitude_flag = 0;  //纬度半球 N（北半球）或 S（南半球）
			double longitude = 0;  //经度
			char longitude_flag = 0;  //经度半球 E（东经）或 W（西经）
			float speed = 0;  //地面速度 节
			float heading = 0;  //航向 度
			char fix_mode = 0;  //定位模式 A=自主定位， D=差分， E=估算， N=数据无效
			char fix_state = 0;  //定位状态， A=有效定位， V=无效定位

			sscanf(p_buffer, "$GNRMC,%*lf,%c,%lf,%c,%lf,%c,%f,%f,%*s", &fix_state, &latitude, &latitude_flag, &longitude, &longitude_flag, &speed, &heading);
			char *p_start = strstr( p_buffer, "*" );
			if( p_start != NULL )
			{
				p_start -= 3;
			}
			sscanf(p_start,"%c,%*s", &fix_mode);

			LOG_INF( "fix_state = %c\r\n", fix_state );
			LOG_INF( "latitude = %10.4lf\r\n", latitude);
			LOG_INF( "latitude_flag = %c\r\n", latitude_flag);
			LOG_INF( "longitude = %10.4lf\r\n", longitude);
			LOG_INF( "longitude_flag = %c\r\n", longitude_flag);
			LOG_INF( "speed = %10.4lf\r\n", speed);
			LOG_INF( "heading = %10.4lf\r\n", heading);
			LOG_INF( "fix_mode = %c\r\n", fix_mode );
			
			pvt->latitude = latitude;
			pvt->longitude = longitude;
			pvt->speed = speed;
			pvt->heading = heading;
			#endif

			#if 1
			//解析日期
			int day, month, year;
			sscanf(p_buffer, "$GNRMC,%*lf,%*c,%*lf,%*c,%*lf,%*c,%*f,%*f,%02d%02d%02d,%*s", &day, &month, &year);
			LOG_INF( "date = %02d-%02d-%02d\r\n",day, month, year );

			//填充解析出的数据到pvt
			pvt->datetime.year = (uint16_t)year;
			pvt->datetime.month = (uint8_t)month;
			pvt->datetime.day = (uint8_t)day;
			#endif

			#if 1
			//解析时间
			int hour, minute, second, millisecond;
			sscanf(p_buffer, "$GNRMC,%02d%02d%02d.%03d,%*c,%*lf,%*c,%*lf,%*c,%*f,%*f,%*s", &hour, &minute, &second, &millisecond);
			LOG_INF( "utc time = %02d:%02d:%02d:%03d\r\n", hour, minute, second, millisecond );

			//填充解析出的数据到pvt
			pvt->datetime.hour = (uint8_t)hour;
			pvt->datetime.minute = (uint8_t)minute;
			pvt->datetime.seconds = (uint8_t)second;
			pvt->datetime.ms = (uint16_t)millisecond;
			#endif
		}
		// $GNVTG  //地面速度信息
		//$GNVTG,74.74,T,,M,0.32,N,0.59,K,A*1E\r\n
		//$GNVTG 语句的基本格式如下：$GNVTG,(1),T,(2),M,(3),N,(4),K,(5)*hh(CR)(LF)
		//(1) 以真北为参考基准的地面航向（000~359 度，前面的 0 也将被传输）(2) 以磁北为参考基准的地面航向(000~359 度，前面的 0 也将被传输)
		//(3) 地面速率(000.0~999.9 节，前面的 0 也将被传输)(4) 地面速率(0000.0~1851.8 公里/小时，前面的 0 也将被传输)(5) 模式指示（A=自主定位， D=差分， E=估算， N=数据无效）
		else if ( strncmp( p_buffer, "$GNVTG", 6) == 0 )
		{
			#if 1
			float heading = 0;  //航向 度
			float speed1 = 0;  //地面速度 节
			float speed2 = 0;  //地面速度 公里/小时
			char fix_mode = 0;  //定位模式 A=自主定位， D=差分， E=估算， N=数据无效
			sscanf(p_buffer, "$GNVTG,%f,%*[^M]M,%f,%*c,%f,%*c,%c,%*s", &heading, &speed1,  &speed2, &fix_mode);

			LOG_INF( "heading = %5.2f\r\n", heading );
			LOG_INF( "speed1 = %5.2f\r\n", speed1 );
			LOG_INF( "speed2 = %5.2f\r\n", speed2 );
			LOG_INF( "fix_mode = %c\r\n", fix_mode );

			pvt->heading = heading;
			pvt->speed = speed2;
			//pvt->speed /= 3600;  //切换单位
			#endif
		}
		//当前时间信息
		//$GNZDA,094117.000,18,09,2023,,*41/r/n
		//$GNZDA 语句的基本格式如下：$GNZDA,(1),(2),(3),(4), (5), (6)*hh(CR)(LF)
		//(1) UTC 时间：hhmmss（时分秒）(2) 日(3) 月(4) 年(5) 本地区域小时（NEO-6M 未用到，为 00）(6) 本地区域分钟（NEO-6M 未用到，为 00）
		else if ( strncmp( p_buffer, "$GNZDA", 6) == 0 )
		{
			#if 1
			//解析日期
			int day, month, year;
			sscanf(p_buffer, "$GNZDA,%*lf,%d,%d,%d", &day, &month, &year);
			LOG_INF( "date = %02d-%02d-%04d\r\n",day, month, year );

			//填充数据到nRF Cloud结构体pvt
			pvt->datetime.year = (uint16_t)year;
			pvt->datetime.month = (uint8_t)month;
			pvt->datetime.day = (uint8_t)day;
			#endif

			#if 1
			//解析时间
			int hour, minute, second, millisecond;
			sscanf(p_buffer, "$GNZDA,%02d%02d%02d.%03d,%*s", &hour, &minute, &second, &millisecond);
			LOG_INF( "utc time = %02d:%02d:%02d:%03d\r\n", hour, minute, second, millisecond );

			//填充数据到nRF Cloud结构体pvt
			pvt->datetime.hour = (uint8_t)hour;
			pvt->datetime.minute = (uint8_t)minute;
			pvt->datetime.seconds = (uint8_t)second;
			pvt->datetime.ms = (uint16_t)millisecond;
			#endif
		}
	}
	#endif  //解析GPS数据结束

	err = 0;
	return err;
}

//需要完善：自动连接到网络指令->连接到nRF Cloud指令->接收串口数据然后转给解析指令
//测试指令：
//AT+CFUN=1  //等待10秒左右获取到IP
//AT#XNRFCLOUD=1  //等待连接上nRF Cloud
//AT#XPTGPS=1,2,3,4,5,6,7  //发送当前GPS坐标到nRF Cloud
//在nRF Cloud查看数据
/*
"Received":{4 items
"appId":"GNSS"
"messageType":"DATA"
"ts":1708335168126
"data":{6 items
"lng":113.570824
"lat":22.325506
"acc":4.531196675286799e-40
"alt":0.472000002861023
"spd":0.5899999737739563
"hdg":74.73999786376953}
}
*/
//实际上传到云端的数据：nrf-351358811387676	GNSS	2023-09-25T08:32:59.857Z	11.53027344	22.54365721	113.9512078	"{acc"":11.5302734375"	alt:-5.53460693359375	spd:0.18539497256278992	"hdg:0}"""	2023-09-25T08:32:59.857Z
int send_ptgps_to_nrfcloud(void)
{
	int err = -EINVAL;
	err = 0;
	
	LOG_INF(" send ptgps data to nrf cloud\r\n");
	//设置p_at3352_gnss_buffer指向串口获取到的GNSS数据
	slm_at_read_ag3352_gnss_data();

	//解析gps_buffer数据为pvt数据
	struct nrf_modem_gnss_pvt_data_frame pvt;
	ag3352_gnss_data_pvt_data( p_at3352_gnss_buffer, &pvt );

	//转换经纬度与海拔高度为正常范围
	pvt.latitude /= 100;
	pvt.longitude /= 100;
	pvt.altitude /= 100;

	//回复定位数据给at串口
	/* GIS accuracy: http://wiki.gis.com/wiki/index.php/Decimal_degrees, use default .6lf */
	rsp_send("\r\n#XGPS: %lf,%lf,%f,%f,%f,%f,\"%04u-%02u-%02u %02u:%02u:%02u\"\r\n",
		pvt.latitude, pvt.longitude, pvt.altitude,
		pvt.accuracy, pvt.speed, pvt.heading,
		pvt.datetime.year, pvt.datetime.month, pvt.datetime.day,
		pvt.datetime.hour, pvt.datetime.minute, pvt.datetime.seconds);

	//打印定位数据
	LOG_INF("\r\n#XGPS: %lf,%lf,%f,%f,%f,%f,\"%04u-%02u-%02u %02u:%02u:%02u\"\r\n",
		pvt.latitude, pvt.longitude, pvt.altitude,
		pvt.accuracy, pvt.speed, pvt.heading,
		pvt.datetime.year, pvt.datetime.month, pvt.datetime.day,
		pvt.datetime.hour, pvt.datetime.minute, pvt.datetime.seconds);

	//打印卫星信息
	for (int i = 0; i < NRF_MODEM_GNSS_MAX_SATELLITES; ++i) 
	{
		if (pvt.sv[i].sv) 
		{ 
			/* SV number 0 indicates no star */
			LOG_INF("SV:%3d sig: %d c/n0:%4d el:%3d az:%3d in-fix: %d unhealthy: %d",
				pvt.sv[i].sv, pvt.sv[i].signal, pvt.sv[i].cn0,
				pvt.sv[i].elevation, pvt.sv[i].azimuth,
				(pvt.sv[i].flags & NRF_MODEM_GNSS_SV_FLAG_USED_IN_FIX) ? 1 : 0,
				(pvt.sv[i].flags & NRF_MODEM_GNSS_SV_FLAG_UNHEALTHY) ? 1 : 0);
		}
	}
	
#if defined(CONFIG_SLM_NRF_CLOUD)
	LOG_INF( "slm_nrf_cloud_ready = %d\r\n", slm_nrf_cloud_ready );
	if (slm_nrf_cloud_ready) 
	{
		/* Report to nRF Cloud by best-effort */
		//发送定位数据到云端
		send_location(&pvt);
	}
#endif /* CONFIG_SLM_NRF_CLOUD */

	return err;
}

//at指令类型 
//AT#XPTGPS=?  AT_CMD_TYPE_TEST_COMMAND
//AT#XPTGPS?  AT_CMD_TYPE_READ_COMMAND
//AT#XPTGPS=1  AT_CMD_TYPE_SET_COMMAND
/* Handles AT#XAG3352 commands. */
int handle_at_ag3352(enum at_cmd_type cmd_type)
{
	int err = -EINVAL;

	switch (cmd_type) 
	{
		case AT_CMD_TYPE_SET_COMMAND:
			rsp_send("\r\nAT_CMD_TYPE_SET_COMMAND\r\n");
			cmd_test();
			//err = send_ptgps_to_nrfcloud();
			err = 0;
			break;
		case AT_CMD_TYPE_READ_COMMAND:
			rsp_send("\r\nAT_CMD_TYPE_READ_COMMAND\r\n");
			err = 0;
			break;
		case AT_CMD_TYPE_TEST_COMMAND:
			rsp_send("\r\nAT_CMD_TYPE_TEST_COMMAND\r\n");
			err = 0;
			break;
		default: 
			break;
	}
	return err;
}

//at指令类型
//AT#XPTGPS=?  AT_CMD_TYPE_TEST_COMMAND
//AT#XPTGPS?  AT_CMD_TYPE_READ_COMMAND
//AT#XPTGPS=1  AT_CMD_TYPE_SET_COMMAND
/* Handles AT#XPTGPSDEL commands. */
int handle_at_ag3352_delete(enum at_cmd_type cmd_type)
{
	int err = -EINVAL;

	switch (cmd_type) 
	{
		case AT_CMD_TYPE_SET_COMMAND:
			rsp_send("\r\nAT_CMD_TYPE_SET_COMMAND\r\n");
			break;
		case AT_CMD_TYPE_READ_COMMAND:
			rsp_send("\r\nAT_CMD_TYPE_READ_COMMAND\r\n");
			err = 0;
			break;
		case AT_CMD_TYPE_TEST_COMMAND:
			rsp_send("\r\nAT_CMD_TYPE_TEST_COMMAND\r\n");
			err = 0;
			break;
		default:
			break;
	}

	return err;
}

/**@brief API to initialize GNSS AT commands handler
 */
int slm_at_ag3352_init(void)
{
	int err = 0;
	//初始化PTGPS FIFO 
	
	return err;
}

/**@brief API to uninitialize GNSS AT commands handler
 * 
*/

int slm_at_ag3352_uninit(void)
{
	return 0;
}
